/* Stdlib */
#include "pico/stdlib.h"

/* App */
#include "app_sensors_impl.h"

int app_sensors_impl_init(void *pdev) {
    app_sensors_impl_t *impl = pdev;

    i2c_init(impl->i2c_inst, 400000);

    gpio_set_function(impl->sda_pin, GPIO_FUNC_I2C);
    gpio_set_function(impl->scl_pin, GPIO_FUNC_I2C);

    gpio_pull_up(impl->sda_pin);
    gpio_pull_up(impl->scl_pin);

    if (impl->led_pin != 0xFF) {
        gpio_init(impl->led_pin);
        gpio_set_dir(impl->led_pin, GPIO_OUT);
    }

    return 0;
}

ims_ret_t app_sensors_i2c_xfer(void *pdev, ims_i2c_xfer_desc_t *xfer) {
    app_sensors_impl_t *impl = pdev;

    gpio_put(impl->led_pin, 1U);

    if (xfer->tx_data) {
        bool no_stop = false;

        if (xfer->rx_data) {
            no_stop = true;
        }

        int ret = i2c_write_blocking(impl->i2c_inst, impl->dev_addr, xfer->tx_data, xfer->tx_size, no_stop);
        if (ret != xfer->tx_size) {
            return IMS_FAIL;
        }
    }

    if (xfer->rx_data) {
        int ret = i2c_read_blocking(impl->i2c_inst, impl->dev_addr, xfer->rx_data, xfer->rx_size, false);
        if (ret != xfer->rx_size) {
            return IMS_FAIL;
        }
    }

    gpio_put(impl->led_pin, 0U);

    return IMS_SUCCESS;
}

ims_ret_t app_sensors_delay(void *pdev, uint32_t delay_msec) {
    sleep_ms(delay_msec);

    return IMS_SUCCESS;
}