#include <stdio.h>

/* Stdlib */
#include "pico/stdlib.h"

/* Sensors */
#include "imsensors/franklin/franklin_as3935.h"

/* App */
#include "app_sensors_impl.h"

#define APP_SENSORS_I2C_INST    PICO_DEFAULT_I2C_INSTANCE
#define APP_SENSORS_I2C_SCL_PIN PICO_DEFAULT_I2C_SCL_PIN
#define APP_SENSORS_I2C_SDA_PIN PICO_DEFAULT_I2C_SDA_PIN
#define APP_SENSORS_LTN_IRQ_PIN 6

static app_sensors_impl_t s_sensor_impl = {
    .i2c_inst = APP_SENSORS_I2C_INST,
    .sda_pin  = APP_SENSORS_I2C_SDA_PIN,
    .scl_pin  = APP_SENSORS_I2C_SCL_PIN,
    .led_pin  = PICO_DEFAULT_LED_PIN,

    .dev_addr = 0x02U,
};

static ims_as3935_t s_ltn = {
    .pdev = &s_sensor_impl,
    .cb =
        {
            .i2c_xfer = app_sensors_i2c_xfer,
            .delay    = app_sensors_delay,
        },
};

static volatile bool s_irq;

static void app_ltn_irq_callback(uint gpio, uint32_t event_mask);

int main(void) {
    stdio_init_all();

    app_sensors_impl_init(&s_sensor_impl);

    printf("Sensor interface initialized.\r\n");

    gpio_set_irq_enabled_with_callback(APP_SENSORS_LTN_IRQ_PIN, GPIO_IRQ_EDGE_RISE, true, app_ltn_irq_callback);

    ims_as3935_init(&s_ltn);

    s_irq = false;

    for (;;) {
        while (s_irq == false) {
            //
        }

        uint8_t irq_status;
        ims_as3935_get_irq_status(&s_ltn, &irq_status);

        printf("[%8ld] ", to_ms_since_boot(get_absolute_time()));

        if (irq_status & IMS_AS3935_IRQ_TYPE_NH) {
            printf("Noise level too high...\r\n");
            uint8_t nf;

            ims_as3935_get_noise_floor(&s_ltn, &nf);
            printf("Current NF: %d\r\n", nf);

            if (nf < 7) {
                nf++;

                printf("Increasing NF to %d\r\n", nf);
                ims_as3935_set_noise_floor(&s_ltn, nf);
            }
        }

        if (irq_status & IMS_AS3935_IRQ_TYPE_D) {
            printf("Disturber detected...\r\n");
        }

        if (irq_status & IMS_AS3935_IRQ_TYPE_L) {
            printf("Lightning detected...\r\n");
        }

        s_irq = false;
    }

    return 0;
}

static void app_ltn_irq_callback(uint gpio, uint32_t event_mask) {
    gpio_acknowledge_irq(gpio, event_mask);

    s_irq = true;
}